@@ -135,8 +135,8 @@ BEGIN {
     $ipv6_addr_len = undef;
     $SNMP_Session::ipv6available = 0;
 
-    if (eval {require Socket6;} &&
-	eval {require IO::Socket::INET6; IO::Socket::INET6->VERSION("1.26");}) {
+    if (eval {local $SIG{__DIE__};require Socket6;} &&
+	eval {local $SIG{__DIE__};require IO::Socket::INET6; IO::Socket::INET6->VERSION("1.26");}) {
 	import Socket6;
 	$ipv6_addr_len = length(pack_sockaddr_in6(161, inet_pton(AF_INET6(), "::1")));
 	$SNMP_Session::ipv6available = 1;
